==> pwnscripts-0.2.1-py3-none-any.whl: pwnscripts/context.py <==
'''A redefinition of pwntools' `context` to add on
some additional context attributes.'''
from os import path
from pwnlib import context
from pwnlib.log import getLogger
# IMPORTANT: pwnscripts must not be a relative import to prevent circular importing
import pwnscripts
log = getLogger('pwnlib.exploit')

class ContextType(context.ContextType):
	'''This is the extended class that inherits from
	pwnlib.context.ContextType. You can use it to spawn a
	_new_ context, but a new `ContextType()` will not affect
	pwnlib's original internal `pwnlib.context.context`,
	and consequently will not provide expected behaviour.

	Basically, don't init this object unless you know what you're doing.
	'''
	# Waiting for python3.9's dict unions here...
	defaults = {**context.ContextType.defaults,
				**{'libc_database': None, 'libc': None}}
	@context._validator
	def libc_database(self, db_dir: str):
		'''
		>>> context.libc_database = 'libc-database'
		>>> context.libc_database
		<pwnscripts.libcdb_query.libc_database object at 0x7fffffffffff>
		'''
		return pwnscripts.libc_database(db_dir)
	
	@context._validator
	def libc(self, assigned:str):
		'''
		>>> context.libc_database = 'libc-database'
		>>> context.libc = 'examples/libc.so.6'
		[*] '/path/to/pwnscripts/libc-database/db/libc6_2.27-3ubuntu1_amd64.so'
		Arch:     amd64-64-little
		RELRO:    Partial RELRO
		Stack:    Canary found
		NX:       NX enabled
		PIE:      PIE enabled
		>>> context.libc
		libc('/path/to/pwnscripts/libc-database/db/libc6_2.27-3ubuntu1_amd64.so')
		or 
		>>> context.libc = 'libc6_2.27-3ubuntu1_amd64'
		[*] libc=`'libc6_2.27-3ubuntu1_amd64'' is not a file; assuming a libc-database id was given!
		>>> context.libc
		libc('/path/to/pwnscripts/libc-database/db/libc6_2.27-3ubuntu1_amd64.so')
		'''
		if type(assigned) != str:	# assume a libc() object was given
			return assigned
		if path.isfile(assigned):	# assume binary
			return pwnscripts.libc(binary=assigned)
		# assume id
		log.info("libc=`%r' is not a file; assuming a libc-database id was given!" % assigned)
		return pwnscripts.libc(id=assigned)

_pwntools_context = context.context
_pwnscripts_context = ContextType()
_pwnscripts_LOCALS = ['libc_database', 'libc']
class ContextWrapper(ContextType):
	'''Wrapper over pwnlib.context.context so that modifications to
	pwnscripts.context.context will propagate correctly to the rest of pwnlib
	
	This wrapper is highly prone to unexpected behaviour. If you're reading this
	and you have a better programmatic suggestion, please raise an Issue/PullRequest!
	'''
	def __setattr__(self, attr, value):
		if attr not in _pwnscripts_LOCALS:
			return setattr(_pwntools_context, attr, value)
		return setattr(_pwnscripts_context, attr, value)

	def __getattribute__(self, attr):
		if attr not in _pwnscripts_LOCALS:
			return getattr(_pwntools_context, attr)
		return getattr(_pwnscripts_context, attr)

context = ContextWrapper()

==> pwnscripts-0.3.0-py3-none-any.whl: pwnscripts/context.py <==
'''A redefinition of pwntools' `context` to add on
some additional context attributes.'''
from os import path
from pwnlib import context
from pwnlib.log import getLogger
from pwnlib.elf.elf import ELF
# IMPORTANT: pwnscripts must not be a relative import to prevent circular importing
import pwnscripts
log = getLogger('pwnlib.exploit')

_pwntools_context = context.context
_pwnscripts_LOCALS = ['libc_database', 'libc', 'binary', 'clear']

class ContextType(context.ContextType):
	'''This is the extended class that inherits from
	pwnlib.context.ContextType. You can use it to spawn a
	_new_ context, but a new `ContextType()` will not affect
	pwnlib's original internal `pwnlib.context.context`,
	and consequently will not provide expected behaviour.

	Basically, don't init this object unless you know what you're doing.
	'''
	# Waiting for python3.9's dict unions here...
	defaults = {**context.ContextType.defaults,
				**{'libc_database': None, 'libc': None, 'binary': None}}
	
	def clear(self, *a, **kw):
		'''overwritten pwnscripts method: clear pwnscripts context as well
		'''
		self._tls._current.clear()
		super().clear(*a, **kw)
	@context._validator
	def binary(self, binary):
		'''overwritten pwnscripts method: spawn context.binary with pwnscript's ELF()
		'''
		# Some parts of pwnlib make use of context.binary, so we need to write to _pwntools_context
		# This breaks the original design of ContextType, but no better solution has been concieved
		_pwntools_context.binary = binary
		if not isinstance(binary, ELF):	# This is pwnlib's ELF
			import pwnscripts.elf
			binary = pwnscripts.elf.ELF(binary)
		return binary
		''' Future idea feature:
		context.libc = # Some really obscure libc version
		# Setting the context should automagically make process() spawn with the correct ld-linux
		r = context.binary.process()
		# Is this impossible? Maybe.
		'''

	@context._validator
	def libc_database(self, db_dir: str):
		'''
		>>> context.libc_database = 'libc-database'
		>>> context.libc_database
		<pwnscripts.libcdb_query.libc_database object at 0x7fffffffffff>
		'''
		return pwnscripts.libc_database(db_dir)
	
	@context._validator
	def libc(self, assigned:str):
		'''
		>>> context.libc_database = 'libc-database'
		>>> context.libc = 'examples/libc.so.6'
		[*] '/path/to/pwnscripts/libc-database/db/libc6_2.27-3ubuntu1_amd64.so'
		Arch:     amd64-64-little
		RELRO:    Partial RELRO
		Stack:    Canary found
		NX:       NX enabled
		PIE:      PIE enabled
		>>> context.libc
		libc('/path/to/pwnscripts/libc-database/db/libc6_2.27-3ubuntu1_amd64.so')
		or 
		>>> context.libc = 'libc6_2.27-3ubuntu1_amd64'
		[*] libc=`'libc6_2.27-3ubuntu1_amd64'' is not a file; assuming a libc-database id was given!
		>>> context.libc
		libc('/path/to/pwnscripts/libc-database/db/libc6_2.27-3ubuntu1_amd64.so')
		'''
		if type(assigned) != str:	# assume a libc() object was given
			return assigned
		if path.isfile(assigned):	# assume binary
			return pwnscripts.libc(binary=assigned)
		# assume id
		log.info("libc=`%r' is not a file; assuming a libc-database id was given!" % assigned)
		return pwnscripts.libc(id=assigned)

_pwnscripts_context = ContextType()
class ContextWrapper(ContextType):
	'''Wrapper over pwnlib.context.context so that modifications to
	pwnscripts.context.context will propagate correctly to the rest of pwnlib
	
	This wrapper is highly prone to unexpected behaviour. If you're reading this
	and you have a better programmatic suggestion, please raise an Issue/PullRequest!
	'''
	def __setattr__(self, attr, value):
		if attr not in _pwnscripts_LOCALS:
			return setattr(_pwntools_context, attr, value)
		return setattr(_pwnscripts_context, attr, value)

	def __getattribute__(self, attr):
		if attr not in _pwnscripts_LOCALS:
			return getattr(_pwntools_context, attr)
		return getattr(_pwnscripts_context, attr)

context = ContextWrapper()
